
# q-expansion of newform 6336.2.a.i, downloaded from the LMFDB on 17 June 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 6336
weight = 2
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[0], [0], [-3], [-2], [1], [4], [-6], [8], [-3], [0], [-5], [1], [0], [-10], [0], [-6], [-3], [4], [-1], [15], [-4], [-2], [-6], [9], [-7], [18], [-8], [-6], [-2], [15], [16], [6], [-9], [14], [6], [10], [-5], [-4], [-12], [18], [9], [13], [-21], [20], [6], [-8], [20], [-17], [-6], [13], [24], [6], [8], [9], [18], [18], [-6], [-20], [10], [18], [-4], [0], [-16], [12], [-1], [33], [-7], [2], [-12], [-14], [21], [-36], [19], [10], [29], [-27], [-27], [34], [-18], [2], [-12], [10], [18], [29], [-8], [21], [-3], [-28], [12], [-23], [-3], [-12], [-29], [-24], [-4], [-30], [-21], [27], [8], [16], [8], [-18], [36], [0], [44], [17], [12], [36], [0], [26], [-14], [-32], [-18], [17], [43], [-39], [-13], [3], [3], [42], [-17], [-34], [-42], [-48], [-1], [18], [37], [45], [-17], [4], [-34], [12], [-35], [22], [-36], [44], [42], [32], [9], [24], [38], [30], [43], [36], [19], [-39], [-38], [-24], [-25], [48], [52], [27], [-4], [30], [-4], [12], [34], [-18], [32], [-30], [-27], [-42], [16], [15], [-45], [45], [16], [10], [38], [51], [6], [58], [-24], [8], [-23], [-33], [-10], [-33], [-20], [-44], [-32], [6], [-41], [42], [-9], [-30], [-8], [-40], [-22], [6], [41], [-6], [35], [30], [-36], [21], [-22], [-11], [42], [66], [-12], [22], [-2], [56], [-9], [-3], [37], [12], [0], [8], [-64], [-33], [67], [60], [-18], [-1], [-68], [-60], [48], [-21], [28], [4], [39], [25], [-36], [46], [42], [-24], [-44], [-36], [-5], [-52], [-50], [0], [-25], [-66], [-7], [36], [-9], [15], [15], [-16], [52], [-53], [-24], [12], [52], [-12], [14], [6], [-32], [6], [3], [38], [-6], [-36], [-38], [26], [45], [38], [-44], [-72], [46], [-26], [-6], [8], [-9], [-45], [14], [-78], [-5], [17], [-46], [-32], [32], [31], [63], [46], [-52], [36], [-48], [-23], [12], [10], [-28], [-57], [26], [-78], [25], [-78], [-15], [60], [36], [-42], [10], [-48], [-83], [27], [-42], [38], [38], [48], [-20], [84], [-67], [47], [-57], [79], [-36], [-44], [-24], [-42], [66], [-49], [-48], [-34], [51], [-18], [26], [-12], [-52], [17], [-42], [-47], [-6], [-13], [29], [89], [-72], [84], [34], [-18], [-38], [-72], [-16], [21], [-5], [-36], [-49], [70], [55], [9], [36], [79], [-33], [42], [25], [53], [24], [51], [68], [-37], [78], [28], [82], [-54], [15], [78], [6], [-21], [58], [30], [-15], [-90], [53], [-19], [-24], [70], [-64], [9], [80], [78], [-12], [22], [-41], [36], [-54], [14], [30], [-22], [-42], [-15], [22], [-30], [-88], [-27], [-80], [-77], [-88], [93], [-19], [-45], [-63], [41], [69], [32], [10], [18], [-76], [-18], [-50], [39], [-32], [-6], [60], [-38], [58], [84], [68], [15], [26], [66], [-60], [38], [-58], [69], [-24], [-77], [-6], [66], [51], [-56], [-48], [-90], [74], [63], [39], [66], [29], [48]]
